# MoTraS Algorithms

`motras-algorithms` provides several algorithms related to variants of modal transitions systems, such as MTS, DMTS, BMTS and PMTS.

## Building and Development

The [gradle build system](http://www.gradle.org/) is used to build, test and package the tool.
After installing `gradle` you can use the following commands:

* `gradle build` - Download all dependencies and compile the sources.
* `gradle run` - Build and the run the tool.
* `gradle distZip` - Build and package the tool.
* `gradle eclipse` - Generate project files for eclipse.

### Project Dependencies 

* `antlr` - [www.antlr3.org](http://www.antlr3.org)
* `apache-commons-cli` - [commons.apache.org](http://commons.apache.org)
* `apache-commons-io` - [commons.apache.org](http://commons.apache.org)
* `apache-commons-math` - [commons.apache.org](http://commons.apache.org)
* `fastutil` - [fastutil.di.unimi.it](http://fastutil.di.unimi.it)
* `guava` - [code.google.com/p/guava-libraries](http://code.google.com/p/guava-libraries)
* `gson` - [code.google.com/p/google-gson](https://code.google.com/p/google-gson/)
* `JUnit 4`

## Runtime Dependencies

`motras-algorithms` requires a recent version of Java, namely Java 8.

Furthermore some algorithms, such as the `ModalRefinementQSat` and the `LTLChecker` need external applications, which need to be placed in the `lib` subdirectory. The name of executable has to match exactly the names listed below. More specifically:

### ModalRefinementQSat

* `quantor` - [http://fmv.jku.at/quantor/][1]

The recommended version is quantor-3.0 with the picosat-951 back-end. Unfortunately the original sources contain compilation errors. A patched version is available at the motras website.

Different, compiled versions of the tool can be found in `./lib`.

### LTLChecker

* `ltl2ba` - [http://www.lsv.ens-cachan.fr/~gastin/ltl2ba/][2]
* `ltl2dstar` - [http://www.ltl2dstar.de/][3]
* `motras` - [http://anna.fi.muni.cz/~xbenes3/MoTraS/][4]

Compiled versions of the tools can be found in `./lib`.

## Usage

The tool expects the input to be written to `stdin` in the `xmts` file format, which is described here: [xmts-specification][5], and outputs the result to `stdout`. The algorithm is selected via the `-a` flag and additional parameters can be passed via the `-p` flag. The initial states are passed to the algorithms as starting points. Furthermore the arity and supported types of an algorithm can be queried by passing `-q`.

Currently the following algorithms are exposed via the command-line interface.
 
* `CompositionDMTS` (Parameters: synchronizing alphabet for the two systems)
* `CompositionMTS` (Parameters: synchronizing alphabet for the two systems)
* `Conjunction`
* `Consistency`
* `DeterministicHullBMTS`
* `DeterministicHullMTS`
* `LTLChecker` (Parameters: LTL Formula to be checked)
* `MaxImplementation`
* `ModalRefinementFixedPoint`
* `ModalRefinementQSat`
* `Quotient`

### Examples

#### Query Algorithm Information

	[motras-algorithms]$ ./motras-algorithms -q -a ModalRefinementQSat
	Arity: 2
	Supported Types: [mts, dmts, bmts, pmts]

#### Constructing the Deterministic Hull (MTS Version)

Let's assume the following transition system is saved in the file non_deterministic_mts.xmts.

	mts non_deterministic_mts [
		initial_state s1

		action a
		action b
		
		state s1 [
			obligation a.s2 & b.s3
		    
		    a -> s2
		    a -> s3
		    b -> s3
		]
		
		state s2 [
			b -> s4
		]
		
		state s3 [
			a -> s3
			a -> s2
		]
		
		state s4
	]

The deterministic Hull is simply created by piping the file to `stdin` of the tool.

 	[motras-algorithms]$ cat non_deterministic_mts.xmts | ./motras-algorithms -a DeterministicHullMTS 

Which then yields the following result.

	mts deterministicHull[
		initial_state s1
		
		action b
		action a

		state s1[
		    obligation (b.s3 & a.s2_s3)
		    b -> s3 
		    a -> s2_s3 
		]
		
		state s3[
		    a -> s2_s3
		]
		
		state s2_s3[
		    b -> s4 
		    a -> s2_s3 
		]
		
		state s4
    ]

#### Checking Modal Refinement

We now check modal refinement on two systems contained in the traffic examples directory.

    [motras-algorithms]$ cat simple1.xmts param2.xmts | ./motras-algorithms -a ModalRefinementQSat
    true
    [motras-algorithms]$ cat simple1.xmts simple2.xmts | ./motras-algorithms -a ModalRefinementQSat
    false

#### Checking a LTL Formula

This example is taken from [4]. 

    [motras-algorithms]$ cat CoffeeAndTeaDMTS.xmts | ./motras-algorithms -a LTLChecker -p "G ! coffee"
    true

#### Running benchmarks on the Modal Refinement Algorithm

	[motras-algorithms]$ ./motras-algorithms -a ModalRefinementQSat -b ../path/to/benchmark/folder/

[1]: http://fmv.jku.at/quantor/
[2]: http://www.lsv.ens-cachan.fr/~gastin/ltl2ba/ 
[3]: http://www.ltl2dstar.de/
[4]: http://anna.fi.muni.cz/~xbenes3/MoTraS/
[5]: ?
